import datetime

from django.db import models
from django.utils.translation import gettext_lazy as _

from feeds.models import Source

from aleksis.core.models import DashboardWidget

from .util.event_feed import get_current_events_with_cal


class RSSFeedWidget(DashboardWidget):
    template = "dashboardfeeds/rss.html"

    url = models.URLField(verbose_name=_("RSS feed source URL"))
    base_url = models.URLField(
        verbose_name=_("Base URL of related website"),
        help_text=_("The widget will have a link to visit a related website to read more news"),
    )
    text_only = models.BooleanField(
        verbose_name=_("Text only"),
        help_text=_("Do not show an image to depict the news item"),
        default=False,
    )

    rss_source = models.ForeignKey(Source, on_delete=models.CASCADE, editable=False, null=True)

    def save(self, *args, **kwargs):
        # Update the linked RSS source object to transfer data into django-feeds
        if not self.rss_source:
            self.rss_source = Source()
        self.rss_source.name = self.title
        self.rss_source.feed_url = self.url
        self.rss_source.site_url = self.base_url

        self.rss_source.last_success = datetime.datetime.utcnow()
        self.rss_source.last_change = datetime.datetime.utcnow()

        self.rss_source.live = self.active
        self.rss_source.save()

        super().save(*args, **kwargs)

    def get_context(self):
        posts = self.rss_source.posts.all().order_by("-created")
        post = posts[0] if len(posts) > 0 else None
        feed = {
            "title": self.title,
            "url": self.rss_source.feed_url,
            "base_url": self.rss_source.site_url,
            "base_image": self.rss_source.image_url,
            "result": post,
            "hide_image": self.text_only,
        }
        return feed

    class Meta:
        verbose_name = _("RSS Widget")
        verbose_name_plural = _("RSS Widgets")


class ICalFeedWidget(DashboardWidget):
    template = "dashboardfeeds/ical.html"

    url = models.URLField(verbose_name=_("iCalendar URL"))
    base_url = models.URLField(
        verbose_name=_("Base URL of related calendar"),
        help_text=_("The widget will have a link to visit a related website to see more events"),
    )
    events_count = models.IntegerField(verbose_name=_("Number of displayed events"), default=5)

    def get_context(self):
        feed = {
            "base_url": self.base_url,
            "feed_events": get_current_events_with_cal(self.url, self.events_count),
        }
        return feed

    class Meta:
        verbose_name = _("iCalendar Widget")
        verbose_name_plural = _("iCalendar Widgets")



# noqa

from django.contrib import admin

from .models import DashboardWidget, ICalFeedWidget, RSSFeedWidget

admin.site.register(DashboardWidget)
admin.site.register(RSSFeedWidget)
admin.site.register(ICalFeedWidget)



# Generated by Django 3.0.5 on 2020-05-04 12:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0001_initial'),
        ('feeds', '0006_auto_20190901_1644'),
    ]

    operations = [
        migrations.CreateModel(
            name='ICalFeedWidget',
            fields=[
                ('dashboardwidget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.DashboardWidget')),
                ('url', models.URLField(verbose_name='iCalendar URL')),
                ('base_url', models.URLField(help_text='The widget will have a link to visit a related website to see more events', verbose_name='Base URL of related calendar')),
                ('events_count', models.IntegerField(default=5, verbose_name='Number of displayed events')),
            ],
            options={
                'verbose_name': 'iCalendar Widget',
                'verbose_name_plural': 'iCalendar Widgets',
            },
            bases=('core.dashboardwidget',),
        ),
        migrations.CreateModel(
            name='RSSFeedWidget',
            fields=[
                ('dashboardwidget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.DashboardWidget')),
                ('url', models.URLField(verbose_name='RSS feed source URL')),
                ('base_url', models.URLField(help_text='The widget will have a link to visit a related website to read more news', verbose_name='Base URL of related website')),
                ('text_only', models.BooleanField(default=False, help_text='Do not show an image to depict the news item', verbose_name='Text only')),
                ('rss_source', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='feeds.Source')),
            ],
            options={
                'verbose_name': 'RSS Widget',
                'verbose_name_plural': 'RSS Widgets',
            },
            bases=('core.dashboardwidget',),
        ),
    ]



from feeds.utils import update_feeds

from aleksis.core.util.core_helpers import celery_optional


@celery_optional
def get_feeds():
    """Update RSS feeds through django-feeds."""
    return update_feeds(max_feeds=10)



import logging

from django.utils import formats, timezone

import requests
from cache_memoize import cache_memoize
from ics import Calendar

logger = logging.getLogger(__name__)


def get_current_events(calendar: Calendar, limit: int = 5) -> list:
    """Get upcoming events from a calendar (ICS) object.

    :param calendar: The calendar object
    :param limit: Count of events
    :return: List of upcoming events
    """
    i: int = 0
    events: list = []
    for event in calendar.timeline.start_after(timezone.now()):
        # Check for limit
        if i >= limit:
            break
        i += 1

        # Create formatted dates and times for begin and end
        begin_date_formatted = formats.date_format(event.begin)
        end_date_formatted = formats.date_format(event.end)
        begin_time_formatted = formats.time_format(event.begin.time())
        end_time_formatted = formats.time_format(event.end.time())

        if event.begin.date() == event.end.date():
            # Event is only on one day
            formatted = begin_date_formatted

            if not event.all_day:
                # No all day event
                formatted += f" {begin_time_formatted}"

            if event.begin.time != event.end.time():
                # Event has an end time
                formatted += f" – {end_time_formatted}"

        else:
            # Event is on multiple days
            if event.all_day:
                # Event is all day
                formatted = f"{begin_date_formatted} – {end_date_formatted}"
            else:
                # Event has begin and end times
                formatted = (
                    f"{begin_date_formatted} {begin_time_formatted}"
                    " - {end_date_formatted} {end_time_formatted}"
                )

        events.append(
            {
                "name": event.name,
                "begin_timestamp": event.begin.timestamp,
                "end_timestamp": event.end.timestamp,
                "date_formatted": formatted,
            }
        )

    return events


@cache_memoize(300)
def get_current_events_with_cal(calendar_url: str, limit: int = 5) -> list:
    """Get current events.

    Download an iCalendar file from an URL, parse using the ICS library
    and return a limited number of events.
    """
    try:
        content = requests.get(calendar_url, timeout=3)
    except requests.RequestException as e:
        logger.error(str(e))
        return []

    calendar: Calendar = Calendar(content.text)

    return get_current_events(calendar, limit)



from aleksis.core.util.apps import AppConfig


class DefaultConfig(AppConfig):
    name = "aleksis.apps.dashboardfeeds"
    verbose_name = "AlekSIS — Dashboard Feeds (Feeds from external resources as dashboard widgets)"

    urls = {
        "Repository": "https://edugit.org/AlekSIS/official/AlekSIS-App-DashboardFeeds/",
    }
    licence = "EUPL-1.2+"
    copyright_info = (
        ([2020], "Dominik George", "dominik.george@teckids.org"),
        ([2020], "Julian Leucker", "leuckeju@katharineum.de"),
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aleksis',
 'aleksis.apps.dashboardfeeds',
 'aleksis.apps.dashboardfeeds.migrations',
 'aleksis.apps.dashboardfeeds.util']

package_data = \
{'': ['*'],
 'aleksis.apps.dashboardfeeds': ['locale/*',
                                 'locale/ar/LC_MESSAGES/django.po',
                                 'locale/de_DE/LC_MESSAGES/django.po',
                                 'locale/fr/LC_MESSAGES/django.po',
                                 'locale/la/LC_MESSAGES/django.po',
                                 'locale/nb_NO/LC_MESSAGES/django.po',
                                 'locale/tr_TR/LC_MESSAGES/django.po',
                                 'static/*',
                                 'static/dashboardfeeds/*',
                                 'static/dashboardfeeds/css/*',
                                 'templates/dashboardfeeds/*']}

install_requires = \
['AlekSIS>=2.0a2,<3.0',
 'django-feed-reader>=0.2.1,<0.3.0',
 'feedparser>=5.2.1,<6.0.0',
 'ics>=0.7,<0.8']

setup_kwargs = {
    'name': 'aleksis-app-dashboardfeeds',
    'version': '2.0a2',
    'description': 'AlekSIS (School Information System)\u200a—\u200aApp Dashboard Feeds (Include feeds from external resources as widgets on dashboard)',
    'long_description': 'AlekSIS (School Information System)\u200a—\u200aApp Dashboard Feeds (Include feeds from external resources as widgets on dashboard)\n=========================================================================================================================\n\nAlekSIS\n-------\n\nThis is an application for use with the `AlekSIS`_ platform.\n\nFeatures\n--------\n\n* Add RSS widgets to dashboard\n* Add iCal widgets to dashboard\n\nLicence\n-------\n\n::\n\n  Copyright © 2020 Dominik George <dominik.george@teckids.org>\n  Copyright © 2020 Julian Leucker <leuckerj@gmail.com>\n\n  Licenced under the EUPL, version 1.2 or later\n\nPlease see the LICENCE.rst file accompanying this distribution for the\nfull licence text or on the `European Union Public Licence`_ website\nhttps://joinup.ec.europa.eu/collection/eupl/guidelines-users-and-developers\n(including all other official language versions).\n\n.. _AlekSIS: https://aleksis.org/\n.. _European Union Public Licence: https://eupl.eu/\n',
    'author': 'Julian Leucker',
    'author_email': 'leuckerj@gmail.com',
    'maintainer': 'Jonathan Weth',
    'maintainer_email': 'wethjo@katharineum.de',
    'url': 'https://aleksis.org/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
